<?php
/**
* 2007-2021.
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you are unable to obtain it through the world-wide-web, please
* send an email to SIA so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize this module for your special
* needs please send an email to SIA for more information.
*
*  @author    SIA
*  @copyright 2007-2021
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class PosSiaOkModuleFrontController extends ModuleFrontController
{
    public $ssl = true;

    /**
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        $this->display_column_left = false;
        parent::initContent();

        $paymentGateway = $this->module->getConfiguration();

        $rfo = new PosSiaResponseFrontOffice();

        $rfo->setMacKey($paymentGateway->macKeyResponse);

        $rfo->setOrderId(Tools::getValue('ORDERID'));
        $rfo->setShopId(Tools::getValue('SHOPID'));
        $rfo->setAuthNumber(Tools::getValue('AUTHNUMBER'));
        $rfo->setAmount(Tools::getValue('AMOUNT'));
        $rfo->setCurrency(Tools::getValue('CURRENCY'));
        $rfo->setTransactionId(Tools::getValue('TRANSACTIONID'));
        $rfo->setMac(Tools::getValue('MAC'));
        $rfo->setResult(Tools::getValue('RESULT'));
        $rfo->setAuthorMode(Tools::getValue('AUTHORMODE'));
        $rfo->setAccountingMode(Tools::getValue('ACCOUNTINGMODE'));
        $rfo->setNetwork(Tools::getValue('NETWORK'));
        $rfo->setTransactionType(Tools::getValue('TRANSACTIONTYPE'));
        $rfo->setIssuerCountry(Tools::getValue('ISSUER_COUNTRY'));

        
        $rfo->calculateMac();

        if ($rfo->responseVerified() && Tools::strtolower($rfo->getResult()) == '00') {
            $arr_exploded = explode('_', $rfo->getOrderId());

            if (count($arr_exploded) > 1) {
                $id_cart = $arr_exploded[1];
                $cart = new Cart($id_cart);
                $this->context->cart = $cart;

                $sql = new DbQuery();
                $sql->select('*');
                $sql->from('possia_payment_verification');
                $sql->where('cart_id = "'.pSQL((int) $id_cart).'"');
                $sql->where('order_id like "'.pSQL($rfo->getOrderId()).'"');

                $infoPayment = Db::getInstance()->executeS($sql);
                
                if ($infoPayment) {
                    if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 ||
                        $cart->id_address_invoice == 0 || !$this->module->active) {
                        Tools::redirect('index.php?controller=order&step=1');
                    }

                    $customer = new Customer($cart->id_customer);
                    if (!Validate::isLoadedObject($customer)) {
                        Tools::redirect('index.php?controller=order&step=1');
                    }


                    if (Validate::isLoadedObject($this->context->cart) && $this->context->cart->OrderExists() == false) {
                        $authorized = false;
                        foreach (Module::getPaymentModules() as $module) {
                            if ($module['name'] == 'possia') {
                                $authorized = true;
                                break;
                            }
                        }
                        if (!$authorized) {
                            $this->error = 'Not Authorized';
                        }


                        $currency = $this->context->currency;
                        $extraVars = array(
                        'order_id' => $rfo->getOrderId(),
                        );

                        $this->module->validateOrder(
                            $cart->id,
                            Configuration::get('PS_OS_PAYMENT'),
                            $infoPayment[0]['total'],
                            $this->module->displayName,
                            null,
                            $extraVars,
                            (int) $currency->id,
                            false,
                            $customer->secure_key
                        );
                    }
                    $sql = new DbQuery();
                    $sql->select('id_order');
                    $sql->from('orders');
                    $sql->where('id_cart = "'.pSQL((int) $cart->id).'"');
                    $result = Db::getInstance()->executeS($sql);

                    Db::getInstance()->update('possia_payment_verification', array('result'=>'OK', 'transaction_id'=>(string) $rfo->getTransactionId(), 'id_order'=>(int) $result[0]['id_order'], 'auth_number'=>(int)  $rfo->getAuthNumber(), 'state'=>'PAYMENT ACCEPTED'), 'order_id like "'.pSQL($rfo->getOrderId()).'"');
                    
                    Tools::redirect(
                        'index.php?controller=order-confirmation&id_cart='.$cart->id.
                        '&id_module='.$this->module->id.'&id_order='.$this->module->currentOrder.
                        '&key='.$customer->secure_key
                    );
                } else {
                    $this->context->smarty->assign(array('error' => $this->module->l('Cart not found')));
                }
            } else {
                $this->context->smarty->assign(array('error' => $this->module->l('Cart not found')));
            }
        } else {
            $this->setTemplate('module:possia/views/templates/front/payment_error.tpl');
        }
    }
}
